package com.hrsk.cloud.eg.domain.api;

import com.alibaba.cola.domain.Entity;
import com.alibaba.cola.exception.BizException;
import com.hrsk.cloud.eg.domain.api.common.ApiPayloadTypeEnum;
import com.hrsk.cloud.eg.domain.api.resolver.payload.ApiPayloadResolverManage;
import lombok.Getter;
import lombok.ToString;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author: bianlanzhou
 * @create: 2024-07-26 17:59
 * @description: HTTP API
 **/
@ToString
@Entity
public class HttpApi implements Api {
    /**
     * url分隔符
     */
    private static final String URL_SEPARATOR = "/";

    /**
     * search分隔符
     */
    private static final String URL_SEARCH_SEPARATOR = "&";

    /**
     * 等于号
     */
    private static final String _EQUAL = "=";
    /**
     * search标记符
     */
    private static final String URL_SEARCH = "?";

    @Override
    public String getType() {
        return "http";
    }

    /**
     * url地址
     */
    @Getter
    private String url;
    /**
     * 请求方法
     */
    @Getter
    private String method;
    /**
     * http api配置
     */
    private HttpApiConfig httpApiConfig;
    /**
     * header数据
     */
    @Getter
    private final Map<String,Object> header = new HashMap<>();
    /**
     * 请求payload数据
     */
    @Getter
    private Object payload;


    /**
     * 加载
     * @param httpApiConfig http api 配置
     */
    public void load(HttpApiConfig httpApiConfig,Object input){
        this.httpApiConfig = httpApiConfig;
        ApiContext context = new ApiContext(input);
        this.method = httpApiConfig.getMethod();
        loadCompleteUrl(context);
        loadHeader(context);
        loadBody(context);
    }

    /**
     * 加载path,包含path和search
     * @param context 上下文
     */
    private void loadCompleteUrl(ApiContext context){
        StringBuilder urlSb = new StringBuilder(httpApiConfig.getUrl());
        for(ValueObject valueObject : httpApiConfig.getPath()){
            urlSb.append(URL_SEPARATOR).append(valueObject.compile(context).getRealValue());
        }
        if(!CollectionUtils.isEmpty(httpApiConfig.getSearch())){
            StringBuilder searchSb = new StringBuilder();
            for(KeyValueObject keyValueObject : httpApiConfig.getSearch()){
                searchSb.append(URL_SEARCH_SEPARATOR).append(keyValueObject.getKey()).append(_EQUAL).append(keyValueObject.compile(context).getRealValue());
            }
            urlSb.append(URL_SEARCH).append(searchSb.toString().replaceFirst(URL_SEARCH_SEPARATOR,""));
        }
        url = urlSb.toString();
    }

    /**
     * 加载header
     * @param context 上下文
     */
    private void loadHeader(ApiContext context){
        for(KeyValueObject keyValueObject : httpApiConfig.getHeader()){
            this.header.put(keyValueObject.getKey(),keyValueObject.compile(context).getRealValue());
        }
    }




    /**
     * 加载请求体
     * @param context 上下文
     */
    private void loadBody(ApiContext context){
        if(StringUtils.equals(httpApiConfig.getContentType(), ApiPayloadTypeEnum.APPLICATION_JSON.getCode())){
            payload = Objects.toString(ApiPayloadResolverManage.getInstance().resolve(httpApiConfig.getContentType(),httpApiConfig.getContent(),context),"");
        }else{
            throw new BizException("不支持的请求体类型!");
        }
    }


}
